package net.minecraftforge.items.wrapper;

import net.minecraft.inventory.ISidedInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.ItemHandlerHelper;

public class SidedInvWrapper implements IItemHandlerModifiable
{
    protected final ISidedInventory inv;
    protected final EnumFacing side;

    public SidedInvWrapper(ISidedInventory inv, EnumFacing side)
    {
        this.inv = inv;
        this.side = side;
    }

    public static int getSlot(ISidedInventory inv, int slot, EnumFacing side)
    {
        int[] slots = inv.func_180463_a(side);
        if (slot < slots.length)
            return slots[slot];
        return -1;
    }

    @Override
    public boolean equals(Object o)
    {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        SidedInvWrapper that = (SidedInvWrapper) o;

        return inv.equals(that.inv) && side == that.side;
    }

    @Override
    public int hashCode()
    {
        int result = inv.hashCode();
        result = 31 * result + side.hashCode();
        return result;
    }

    @Override
    public int getSlots()
    {
        return inv.func_180463_a(side).length;
    }

    @Override
    public ItemStack getStackInSlot(int slot)
    {
        int i = getSlot(inv, slot, side);
        return i == -1 ? null : inv.func_70301_a(i);
    }

    @Override
    public ItemStack insertItem(int slot, ItemStack stack, boolean simulate)
    {

        if (stack == null)
            return null;

        int slot1 = getSlot(inv, slot, side);

        if (slot1 == -1)
            return stack;

        if (!inv.func_94041_b(slot1, stack) || !inv.func_180462_a(slot1, stack, side))
            return stack;

        ItemStack stackInSlot = inv.func_70301_a(slot1);

        int m;
        if (stackInSlot != null)
        {
            if (!ItemHandlerHelper.canItemStacksStack(stack, stackInSlot))
                return stack;

            m = Math.min(stack.func_77976_d(), inv.func_70297_j_()) - stackInSlot.field_77994_a;

            if (stack.field_77994_a <= m)
            {
                if (!simulate)
                {
                    ItemStack copy = stack.func_77946_l();
                    copy.field_77994_a += stackInSlot.field_77994_a;
                    inv.func_70299_a(slot1, copy);
                }

                return null;
            }
            else
            {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate)
                {
                    ItemStack copy = stack.func_77979_a(m);
                    copy.field_77994_a += stackInSlot.field_77994_a;
                    inv.func_70299_a(slot1, copy);
                    return stack;
                }
                else
                {
                    stack.field_77994_a -= m;
                    return stack;
                }
            }
        }
        else
        {
            m = Math.min(stack.func_77976_d(), inv.func_70297_j_());
            if (m < stack.field_77994_a)
            {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate)
                {
                    inv.func_70299_a(slot1, stack.func_77979_a(m));
                    return stack;
                }
                else
                {
                    stack.field_77994_a -= m;
                    return stack;
                }
            }
            else
            {
                if (!simulate)
                    inv.func_70299_a(slot1, stack);
                return null;
            }
        }

    }

    @Override
    public void setStackInSlot(int slot, ItemStack stack)
    {
        inv.func_70299_a(slot, stack);
    }

    @Override
    public ItemStack extractItem(int slot, int amount, boolean simulate)
    {
        if (amount == 0)
            return null;

        int slot1 = getSlot(inv, slot, side);

        if (slot1 == -1)
            return null;

        ItemStack stackInSlot = inv.func_70301_a(slot1);

        if (stackInSlot == null)
            return null;

        if (!inv.func_180461_b(slot1, stackInSlot, side))
            return null;

        if (simulate)
        {
            if (stackInSlot.field_77994_a < amount)
            {
                return stackInSlot.func_77946_l();
            }
            else
            {
                ItemStack copy = stackInSlot.func_77946_l();
                copy.field_77994_a = amount;
                return copy;
            }
        }
        else
        {
            int m = Math.min(stackInSlot.field_77994_a, amount);
            return inv.func_70298_a(slot1, m);
        }
    }
}
